@@ -40,7 +40,7 @@ def _geo_op(this, other, op):
 # TODO: think about merging with _geo_op
 def _series_op(this, other, op, **kwargs):
     """Geometric operation that returns a pandas Series"""
-    null_val = False if op != 'distance' else np.nan
+    null_val = False if op not in ['distance', 'project'] else np.nan
 
     if isinstance(other, GeoPandasBase):
         this = this.geometry
@@ -48,10 +48,11 @@ def _series_op(this, other, op, **kwargs):
         return Series([getattr(this_elem, op)(other_elem, **kwargs)
                     if not this_elem.is_empty | other_elem.is_empty else null_val
                     for this_elem, other_elem in zip(this, other)],
-                    index=this.index)
+                    index=this.index, dtype=np.dtype(type(null_val)))
     else:
         return Series([getattr(s, op)(other, **kwargs) if s else null_val
-                      for s in this.geometry], index=this.index)
+                       for s in this.geometry],
+                      index=this.index, dtype=np.dtype(type(null_val)))
 
 
 def _geo_unary_op(this, op):
@@ -63,7 +64,7 @@ def _geo_unary_op(this, op):
 def _series_unary_op(this, op, null_value=False):
     """Unary operation that returns a Series"""
     return Series([getattr(geom, op, null_value) for geom in this.geometry],
-                     index=this.index)
+                     index=this.index, dtype=np.dtype(type(null_value)))
 
 
 class GeoPandasBase(object):
@@ -98,12 +99,14 @@ class GeoPandasBase(object):
 
     @property
     def area(self):
-        """Return the area of each geometry in the GeoSeries"""
+        """Returns a ``Series`` containing the area of each geometry in the
+        ``GeoSeries``."""
         return _series_unary_op(self, 'area', null_value=np.nan)
 
     @property
     def geom_type(self):
-        """Return the geometry type of each geometry in the GeoSeries"""
+        """Returns a ``Series`` of strings specifying the `Geometry Type` of each
+        object."""
         return _series_unary_op(self, 'geom_type', null_value=None)
 
     @property
@@ -113,69 +116,106 @@ class GeoPandasBase(object):
 
     @property
     def length(self):
-        """Return the length of each geometry in the GeoSeries"""
+        """Returns a ``Series`` containing the length of each geometry."""
         return _series_unary_op(self, 'length', null_value=np.nan)
 
     @property
     def is_valid(self):
-        """Return True for each valid geometry, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        geometries that are valid."""
         return _series_unary_op(self, 'is_valid', null_value=False)
 
     @property
     def is_empty(self):
-        """Return True for each empty geometry, False for non-empty"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        empty geometries."""
         return _series_unary_op(self, 'is_empty', null_value=False)
 
     @property
     def is_simple(self):
-        """Return True for each simple geometry, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        geometries that do not cross themselves.
+
+        This is meaningful only for `LineStrings` and `LinearRings`.
+        """
         return _series_unary_op(self, 'is_simple', null_value=False)
 
     @property
     def is_ring(self):
-        """Return True for each geometry that is a closed ring, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        features that are closed."""
         # operates on the exterior, so can't use _series_unary_op()
         return Series([geom.exterior.is_ring for geom in self.geometry],
                       index=self.index)
 
+    @property
+    def has_z(self):
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        features that have a z-component."""
+        # operates on the exterior, so can't use _series_unary_op()
+        return _series_unary_op(self, 'has_z', null_value=False)
+
     #
     # Unary operations that return a GeoSeries
     #
 
     @property
     def boundary(self):
-        """Return the bounding geometry for each geometry"""
+        """Returns a ``GeoSeries`` of lower dimensional objects representing
+        each geometries's set-theoretic `boundary`."""
         return _geo_unary_op(self, 'boundary')
 
     @property
     def centroid(self):
-        """Return the centroid of each geometry in the GeoSeries"""
+        """Returns a ``GeoSeries`` of points representing the centroid of each
+        geometry."""
         return _geo_unary_op(self, 'centroid')
 
     @property
     def convex_hull(self):
-        """Return the convex hull of each geometry"""
+        """Returns a ``GeoSeries`` of geometries representing the convex hull
+        of each geometry.
+
+        The convex hull of a geometry is the smallest convex `Polygon`
+        containing all the points in each geometry, unless the number of points
+        in the geometric object is less than three. For two points, the convex
+        hull collapses to a `LineString`; for 1, a `Point`."""
         return _geo_unary_op(self, 'convex_hull')
 
     @property
     def envelope(self):
-        """Return a bounding rectangle for each geometry"""
+        """Returns a ``GeoSeries`` of geometries representing the envelope of
+        each geometry.
+
+        The envelope of a geometry is the bounding rectangle. That is, the
+        point or smallest rectangular polygon (with sides parallel to the
+        coordinate axes) that contains the geometry."""
         return _geo_unary_op(self, 'envelope')
 
     @property
     def exterior(self):
-        """Return the outer boundary of each polygon"""
+        """Returns a ``GeoSeries`` of LinearRings representing the outer
+        boundary of each polygon in the GeoSeries.
+
+        Applies to GeoSeries containing only Polygons.
+        """
         # TODO: return empty geometry for non-polygons
         return _geo_unary_op(self, 'exterior')
 
     @property
     def interiors(self):
-        """Return the interior rings of each polygon"""
+        """Returns a ``GeoSeries`` of InteriorRingSequences representing the
+        inner rings of each polygon in the GeoSeries.
+
+        Applies to GeoSeries containing only Polygons.
+        """
         # TODO: return empty list or None for non-polygons
         return _series_unary_op(self, 'interiors', null_value=False)
 
     def representative_point(self):
-        """Return a GeoSeries of points guaranteed to be in each geometry"""
+        """Returns a ``GeoSeries`` of (cheaply computed) points that are
+        guaranteed to be within each geometry.
+        """
         return gpd.GeoSeries([geom.representative_point()
                              for geom in self.geometry],
                          index=self.index)
@@ -191,7 +231,8 @@ class GeoPandasBase(object):
 
     @property
     def unary_union(self):
-        """Return the union of all geometries"""
+        """Returns a geometry containing the union of all geometries in the
+        ``GeoSeries``."""
         return unary_union(self.geometry.values)
 
     #
@@ -199,33 +240,107 @@ class GeoPandasBase(object):
     #
 
     def contains(self, other):
-        """Return True for all geometries that contain *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that contains `other`.
+
+        An object is said to contain `other` if its `interior` contains the
+        `boundary` and `interior` of the other object and their boundaries do
+        not touch at all.
+
+        This is the inverse of :meth:`within` in the sense that the expression
+        ``a.contains(b) == b.within(a)`` always evaluates to ``True``.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if is
+            contained.
+        """
         return _series_op(self, other, 'contains')
 
     def geom_equals(self, other):
-        """Return True for all geometries that equal *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry equal to `other`.
+
+        An object is said to be equal to `other` if its set-theoretic
+        `boundary`, `interior`, and `exterior` coincides with those of the
+        other.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test for
+            equality.
+        """
         return _series_op(self, other, 'equals')
 
     def geom_almost_equals(self, other, decimal=6):
-        """Return True for all geometries that is approximately equal to *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` if
+        each geometry is approximately equal to `other`.
+
+        Approximate equality is tested at all points to the specified `decimal`
+        place precision.  See also :meth:`equals`.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to compare to.
+        decimal : int
+            Decimal place presion used when testing for approximate equality.
+        """
         # TODO: pass precision argument
         return _series_op(self, other, 'almost_equals', decimal=decimal)
 
     def geom_equals_exact(self, other, tolerance):
-        """Return True for all geometries that equal *other* to a given tolerance, else False"""
+        """Return True for all geometries that equal *other* to a given
+        tolerance, else False"""
         # TODO: pass tolerance argument.
         return _series_op(self, other, 'equals_exact', tolerance=tolerance)
 
     def crosses(self, other):
-        """Return True for all geometries that cross *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that cross `other`.
+
+        An object is said to cross `other` if its `interior` intersects the
+        `interior` of the other but does not contain it, and the dimension of
+        the intersection is less than the dimension of the one or the other.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if is
+            crossed.
+        """
         return _series_op(self, other, 'crosses')
 
     def disjoint(self, other):
-        """Return True for all geometries that are disjoint with *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry disjoint to `other`.
+
+        An object is said to be disjoint to `other` if its `boundary` and
+        `interior` does not intersect at all with those of the other.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if is
+            disjoint.
+        """
         return _series_op(self, other, 'disjoint')
 
     def intersects(self, other):
-        """Return True for all geometries that intersect *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that intersects `other`.
+
+        An object is said to intersect `other` if its `boundary` and `interior`
+        intersects in any way with those of the other.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if is
+            intersected.
+        """
         return _series_op(self, other, 'intersects')
 
     def overlaps(self, other):
@@ -233,15 +348,51 @@ class GeoPandasBase(object):
         return _series_op(self, other, 'overlaps')
 
     def touches(self, other):
-        """Return True for all geometries that touch *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that touches `other`.
+
+        An object is said to touch `other` if it has at least one point in
+        common with `other` and its interior does not intersect with any part
+        of the other.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if is
+            touched.
+        """
         return _series_op(self, other, 'touches')
 
     def within(self, other):
-        """Return True for all geometries that are within *other*, else False"""
+        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that is within `other`.
+
+        An object is said to be within `other` if its `boundary` and `interior`
+        intersects only with the `interior` of the other (not its `boundary` or
+        `exterior`).
+
+        This is the inverse of :meth:`contains` in the sense that the
+        expression ``a.within(b) == b.contains(a)`` always evaluates to
+        ``True``.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if each
+            geometry is within.
+
+        """
         return _series_op(self, other, 'within')
 
     def distance(self, other):
-        """Return distance of each geometry to *other*"""
+        """Returns a ``Series`` containing the distance to `other`.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to find the
+            distance to.
+        """
         return _series_op(self, other, 'distance')
 
     #
@@ -249,19 +400,54 @@ class GeoPandasBase(object):
     #
 
     def difference(self, other):
-        """Return the set-theoretic difference of each geometry with *other*"""
+        """Returns a ``GeoSeries`` of the points in each geometry that
+        are not in `other`.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to find the
+            difference to.
+        """
         return _geo_op(self, other, 'difference')
 
     def symmetric_difference(self, other):
-        """Return the symmetric difference of each geometry with *other*"""
+        """Returns a ``GeoSeries`` of the symmetric difference of points in
+        each geometry with `other`.
+
+        For each geometry, the symmetric difference consists of points in the
+        geometry not in `other`, and points in `other` not in the geometry.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to find the
+            symmetric difference to.
+        """
         return _geo_op(self, other, 'symmetric_difference')
 
     def union(self, other):
-        """Return the set-theoretic union of each geometry with *other*"""
+        """Returns a ``GeoSeries`` of the union of points in each geometry with
+        `other`.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to find the union
+            with.
+        """
         return _geo_op(self, other, 'union')
 
     def intersection(self, other):
-        """Return the set-theoretic intersection of each geometry with *other*"""
+        """Returns a ``GeoSeries`` of the intersection of points in each
+        geometry with `other`.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to find the
+            intersection with.
+        """
         return _geo_op(self, other, 'intersection')
 
     #
@@ -270,7 +456,11 @@ class GeoPandasBase(object):
 
     @property
     def bounds(self):
-        """Return a DataFrame of minx, miny, maxx, maxy values of geometry objects"""
+        """Returns a ``DataFrame`` with columns ``minx``, ``miny``, ``maxx``,
+        ``maxy`` values containing the bounds for each geometry.
+
+        See ``GeoSeries.total_bounds`` for the limits of the entire series.
+        """
         bounds = np.array([geom.bounds for geom in self.geometry])
         return DataFrame(bounds,
                          columns=['minx', 'miny', 'maxx', 'maxy'],
@@ -278,11 +468,12 @@ class GeoPandasBase(object):
 
     @property
     def total_bounds(self):
-        """Return a single bounding box (minx, miny, maxx, maxy) for all geometries
+        """Returns a tuple containing ``minx``, ``miny``, ``maxx``, ``maxy``
+        values for the bounds of the series as a whole.
 
-        This is a shortcut for calculating the min/max x and y bounds individually.
+        See ``GeoSeries.bounds`` for the bounds of the geometries contained in
+        the series.
         """
-
         b = self.bounds
         return np.array((b['minx'].min(),
                          b['miny'].min(),
@@ -295,12 +486,40 @@ class GeoPandasBase(object):
             self._generate_sindex()
         return self._sindex
 
-    def buffer(self, distance, resolution=16):
-        return gpd.GeoSeries([geom.buffer(distance, resolution)
+    def buffer(self, distance, resolution=16, **kwargs):
+        """Returns a ``GeoSeries`` of geometries representing all points within
+        a given `distance` of each geometric object.
+
+        See http://shapely.readthedocs.io/en/latest/manual.html#object.buffer
+        for details.
+
+        Parameters
+        ----------
+        distance : float
+            The radius of the buffer.
+        resolution: float
+            Optional, the resolution of the buffer around each vertex.
+        """
+        return gpd.GeoSeries([geom.buffer(distance, resolution, **kwargs)
                              for geom in self.geometry],
                          index=self.index, crs=self.crs)
 
     def simplify(self, *args, **kwargs):
+        """Returns a ``GeoSeries`` containing a simplified representation of
+        each geometry.
+
+        See http://shapely.readthedocs.io/en/latest/manual.html#object.simplify
+        for details
+
+        Parameters
+        ----------
+        tolerance : float
+            All points in a simplified geometry will be no more than
+            `tolerance` distance from the original.
+        preserve_topology: bool
+            False uses a quicker algorithm, but may produce self-intersecting
+            or otherwise invalid geometries.
+        """
         return gpd.GeoSeries([geom.simplify(*args, **kwargs)
                              for geom in self.geometry],
                       index=self.index, crs=self.crs)
@@ -343,8 +562,10 @@ class GeoPandasBase(object):
             index=self.index, crs=self.crs)
 
     def translate(self, xoff=0.0, yoff=0.0, zoff=0.0):
-        """
-        Shift the coordinates of the GeoSeries.
+        """Returns a ``GeoSeries`` with translated geometries.
+
+        See http://shapely.readthedocs.io/en/latest/manual.html#shapely.affinity.translate
+        for details.
 
         Parameters
         ----------
@@ -352,18 +573,16 @@ class GeoPandasBase(object):
             Amount of offset along each dimension.
             xoff, yoff, and zoff for translation along the x, y, and z
             dimensions respectively.
-
-        See shapely manual for more information:
-        http://toblerity.org/shapely/manual.html#affine-transformations
         """
-
         return gpd.GeoSeries([affinity.translate(s, xoff, yoff, zoff)
                              for s in self.geometry],
             index=self.index, crs=self.crs)
 
     def rotate(self, angle, origin='center', use_radians=False):
-        """
-        Rotate the coordinates of the GeoSeries.
+        """Returns a ``GeoSeries`` with rotated geometries.
+
+        See http://shapely.readthedocs.io/en/latest/manual.html#shapely.affinity.rotate
+        for details.
 
         Parameters
         ----------
@@ -377,18 +596,19 @@ class GeoPandasBase(object):
             object or a coordinate tuple (x, y).
         use_radians : boolean
             Whether to interpret the angle of rotation as degrees or radians
-
-        See shapely manual for more information:
-        http://toblerity.org/shapely/manual.html#affine-transformations
         """
-
         return gpd.GeoSeries([affinity.rotate(s, angle, origin=origin,
             use_radians=use_radians) for s in self.geometry],
             index=self.index, crs=self.crs)
 
     def scale(self, xfact=1.0, yfact=1.0, zfact=1.0, origin='center'):
-        """
-        Scale the geometries of the GeoSeries along each (x, y, z) dimension.
+        """Returns a ``GeoSeries`` with scaled geometries.
+
+        The geometries can be scaled by different factors along each
+        dimension. Negative scale factors will mirror or reflect coordinates.
+
+        See http://shapely.readthedocs.io/en/latest/manual.html#shapely.affinity.scale
+        for details.
 
         Parameters
         ----------
@@ -398,20 +618,18 @@ class GeoPandasBase(object):
             The point of origin can be a keyword 'center' for the 2D bounding
             box center (default), 'centroid' for the geometry's 2D centroid, a
             Point object or a coordinate tuple (x, y, z).
-
-        Note: Negative scale factors will mirror or reflect coordinates.
-
-        See shapely manual for more information:
-        http://toblerity.org/shapely/manual.html#affine-transformations
         """
-
         return gpd.GeoSeries([affinity.scale(s, xfact, yfact, zfact,
             origin=origin) for s in self.geometry], index=self.index,
             crs=self.crs)
 
     def skew(self, xs=0.0, ys=0.0, origin='center', use_radians=False):
-        """
-        Shear/Skew the geometries of the GeoSeries by angles along x and y dimensions.
+        """Returns a ``GeoSeries`` with skewed geometries.
+
+        The geometries are sheared by angles along the x and y dimensions.
+
+        See http://shapely.readthedocs.io/en/latest/manual.html#shapely.affinity.skew
+        for details.
 
         Parameters
         ----------
@@ -425,11 +643,7 @@ class GeoPandasBase(object):
             object or a coordinate tuple (x, y).
         use_radians : boolean
             Whether to interpret the shear angle(s) as degrees or radians
-
-        See shapely manual for more information:
-        http://toblerity.org/shapely/manual.html#affine-transformations
         """
-
         return gpd.GeoSeries([affinity.skew(s, xs, ys, origin=origin,
             use_radians=use_radians) for s in self.geometry],
             index=self.index, crs=self.crs)
@@ -445,7 +659,8 @@ class GeoPandasBase(object):
         Returns
         ------
         A GeoSeries with a MultiIndex. The levels of the MultiIndex are the
-        original index and an integer.
+        original index and a zero-based integer index that counts the 
+        number of single geometries within a multi-part geometry. 
 
         Example
         -------
@@ -473,8 +688,8 @@ class GeoPandasBase(object):
                 idxs = [(idx, 0)]
             index.extend(idxs)
             geometries.extend(geoms)
-        return gpd.GeoSeries(geometries,
-            index=MultiIndex.from_tuples(index)).__finalize__(self)
+        index = MultiIndex.from_tuples(index, names=self.index.names + [None])
+        return gpd.GeoSeries(geometries, index=index).__finalize__(self)
 
 
 class _CoordinateIndexer(_NDFrameIndexer):
@@ -505,14 +720,3 @@ class _CoordinateIndexer(_NDFrameIndexer):
                    ys.stop if ys.stop is not None else ymax)
         idx = obj.intersects(bbox)
         return obj[idx]
-
-
-def _array_input(arr):
-    if isinstance(arr, (MultiPoint, MultiLineString, MultiPolygon)):
-        # Prevent against improper length detection when input is a
-        # Multi*
-        geom = arr
-        arr = np.empty(1, dtype=object)
-        arr[0] = geom
-
-    return arr
