@@ -1,6 +1,7 @@
 import json
 
 import numpy as np
+import pandas as pd
 from pandas import DataFrame, Series
 from shapely.geometry import mapping, shape
 from shapely.geometry.base import BaseGeometry
@@ -155,19 +156,32 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
     @classmethod
     def from_file(cls, filename, **kwargs):
-        """
-        Alternate constructor to create a GeoDataFrame from a file.
+        """Alternate constructor to create a ``GeoDataFrame`` from a file.
 
-        Example:
-            df = geopandas.GeoDataFrame.from_file('nybb.shp')
+        Can load a ``GeoDataFrame`` from a file in any format recognized by
+        `fiona`. See http://toblerity.org/fiona/manual.html for details.
 
-        Wraps geopandas.read_file(). For additional help, see read_file()
+        Parameters
+        ----------
 
+        filename : str
+            File path or file handle to read from. Depending on which kwargs
+            are included, the content of filename may vary. See
+            http://toblerity.org/fiona/README.html#usage for usage details.
+        kwargs : key-word arguments
+            These arguments are passed to fiona.open, and can be used to
+            access multi-layer data, data stored within archives (zip files),
+            etc.
+
+        Examples
+        --------
+
+        >>> df = geopandas.GeoDataFrame.from_file('nybb.shp')
         """
         return geopandas.io.file.read_file(filename, **kwargs)
 
     @classmethod
-    def from_features(cls, features, crs=None):
+    def from_features(cls, features, crs=None, columns=None):
         """
         Alternate constructor to create GeoDataFrame from an iterable of
         features or a feature collection.
@@ -183,6 +197,10 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
               ``__geo_interface__``.
         crs : str or dict (optional)
             Coordinate reference system to set on the resulting frame.
+        columns : list of column names, optional
+            Optionally specify the column names to include in the output frame.
+            This does not overwrite the property names of the input, but can
+            ensure a consistent output format.
 
         Returns
         -------
@@ -215,57 +233,82 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             d = {'geometry': shape(f['geometry']) if f['geometry'] else None}
             d.update(f['properties'])
             rows.append(d)
-        df = GeoDataFrame.from_dict(rows)
+        df = GeoDataFrame(rows, columns=columns)
         df.crs = crs
         return df
 
     @classmethod
-    def from_postgis(cls, sql, con, geom_col='geom', crs=None, index_col=None,
-                     coerce_float=True, params=None):
-        """
-        Alternate constructor to create a GeoDataFrame from a sql query
+    def from_postgis(cls, sql, con, geom_col='geom', crs=None,
+                     hex_encoded=True, index_col=None, coerce_float=True,
+                     params=None):
+        """Alternate constructor to create a ``GeoDataFrame`` from a sql query
         containing a geometry column.
 
-        Example:
-            df = geopandas.GeoDataFrame.from_postgis(con,
-                "SELECT geom, highway FROM roads;")
-
-        Wraps geopandas.read_postgis(). For additional help, see read_postgis()
+        Parameters
+        ----------
+        sql : string
+        con : DB connection object or SQLAlchemy engine
+        geom_col : string, default 'geom'
+            column name to convert to shapely geometries
+        crs : optional
+            Coordinate reference system to use for the returned GeoDataFrame
+        hex_encoded : bool, optional
+            Whether the geometry is in a hex-encoded string. Default is True,
+            standard for postGIS.
+        index_col : string or list of strings, optional, default: None
+            Column(s) to set as index(MultiIndex)
+        coerce_float : boolean, default True
+            Attempt to convert values of non-string, non-numeric objects (like
+            decimal.Decimal) to floating point, useful for SQL result sets
+        params : list, tuple or dict, optional, default: None
+            List of parameters to pass to execute method.
 
+        Examples
+        --------
+        >>> sql = "SELECT geom, highway FROM roads;"
+        >>> df = geopandas.GeoDataFrame.from_postgis(sql, con)
         """
-        return geopandas.io.sql.read_postgis(sql, con, geom_col, crs, index_col,
-                     coerce_float, params)
+
+        df = geopandas.io.sql.read_postgis(
+                sql, con, geom_col=geom_col, crs=crs, hex_encoded=hex_encoded,
+                index_col=index_col, coerce_float=coerce_float, params=params)
+        return df
 
     def to_json(self, na='null', show_bbox=False, **kwargs):
         """
-        Returns a GeoJSON string representation of the GeoDataFrame.
+        Returns a GeoJSON representation of the ``GeoDataFrame`` as a string.
 
         Parameters
         ----------
         na : {'null', 'drop', 'keep'}, default 'null'
-            Indicates how to output missing (NaN) values in the GeoDataFrame
-            * null: output the missing entries as JSON null
-            * drop: remove the property from the feature. This applies to
-                    each feature individually so that features may have
-                    different properties
-            * keep: output the missing entries as NaN
-
-        show_bbox : include bbox (bounds) in the geojson
+            Indicates how to output missing (NaN) values in the GeoDataFrame.
+            See below.
+        show_bbox : bool, optional, default: False
+            Include bbox (bounds) in the geojson
 
+        Notes
+        -----
         The remaining *kwargs* are passed to json.dumps().
 
+        Missing (NaN) values in the GeoDataFrame can be represented as follows:
+
+        - ``null``: output the missing entries as JSON null.
+        - ``drop``: remove the property from the feature. This applies to each
+          feature individually so that features may have different properties.
+        - ``keep``: output the missing entries as NaN.
         """
         return json.dumps(self._to_geo(na=na, show_bbox=show_bbox), **kwargs)
 
     @property
     def __geo_interface__(self):
-        """
-        Returns a python feature collection (i.e. the geointerface)
-        representation of the GeoDataFrame.
+        """Returns a ``GeoDataFrame`` as a python feature collection.
+
+        Implements the `geo_interface`. The returned python data structure
+        represents the ``GeoDataFrame`` as a GeoJSON-like
+        ``FeatureCollection``.
 
         This differs from `_to_geo()` only in that it is a property with
         default args instead of a method
-
         """
         return self._to_geo(na='null', show_bbox=True)
 
@@ -341,10 +384,12 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
     def to_file(self, filename, driver="ESRI Shapefile", schema=None,
                 **kwargs):
-        """
-        Write this GeoDataFrame to an OGR data source
+        """Write the ``GeoDataFrame`` to a file.
+
+        By default, an ESRI shapefile is written, but any OGR data source
+        supported by Fiona can be written. A dictionary of supported OGR
+        providers is available via:
 
-        A dictionary of supported OGR providers is available via:
         >>> import fiona
         >>> fiona.supported_drivers
 
@@ -352,32 +397,44 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         ----------
         filename : string
             File path or file handle to write to.
-        driver : string, default 'ESRI Shapefile'
+        driver : string, default: 'ESRI Shapefile'
             The OGR format driver used to write the vector file.
-        schema : dict, default None
+        schema : dict, default: None
             If specified, the schema dictionary is passed to Fiona to
             better control how the file is written.
 
-        The *kwargs* are passed to fiona.open and can be used to write
-        to multi-layer data, store data within archives (zip files), etc.
+        Notes
+        -----
+        The extra keyword arguments ``**kwargs`` are passed to fiona.open and
+        can be used to write to multi-layer data, store data within archives
+        (zip files), etc.
         """
         from geopandas.io.file import to_file
         to_file(self, filename, driver, schema, **kwargs)
 
     def to_crs(self, crs=None, epsg=None, inplace=False):
-        """Transform geometries to a new coordinate reference system
+        """Transform geometries to a new coordinate reference system.
 
-        This method will transform all points in all objects.  It has
-        no notion or projecting entire geometries.  All segments
-        joining points are assumed to be lines in the current
-        projection, not geodesics.  Objects crossing the dateline (or
-        other projection boundary) will have undesirable behavior.
+        Transform all geometries in a GeoSeries to a different coordinate
+        reference system.  The ``crs`` attribute on the current GeoSeries must
+        be set.  Either ``crs`` in string or dictionary form or an EPSG code
+        may be specified for output.
 
-        `to_crs` passes the `crs` argument to the `Proj` function from the
-        `pyproj` library (with the option `preserve_units=True`). It can
-        therefore accept proj4 projections in any format
-        supported by `Proj`, including dictionaries, or proj4 strings.
+        This method will transform all points in all objects.  It has no notion
+        or projecting entire geometries.  All segments joining points are
+        assumed to be lines in the current projection, not geodesics.  Objects
+        crossing the dateline (or other projection boundary) will have
+        undesirable behavior.
 
+        Parameters
+        ----------
+        crs : dict or str
+            Output projection parameters as string or in dictionary form.
+        epsg : int
+            EPSG code specifying output projection.
+        inplace : bool, optional, default: False
+            Whether to return a new GeoDataFrame or do the transformation in
+            place.
         """
         if inplace:
             df = self
@@ -465,7 +522,15 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         return GeoDataFrame(data).__finalize__(self)
 
     def plot(self, *args, **kwargs):
+        """Generate a plot of the geometries in the ``GeoDataFrame``.
 
+        If the ``column`` parameter is given, colors plot according to values
+        in that column, otherwise calls ``GeoSeries.plot()`` on the
+        ``geometry`` column.
+
+        Wraps the ``plot_dataframe()`` function, and documentation is copied
+        from there.
+        """
         return plot_dataframe(self, *args, **kwargs)
 
     plot.__doc__ = plot_dataframe.__doc__
@@ -518,6 +583,43 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
         return aggregated
 
+    # overrides GeoPandasBase method
+    def explode(self):
+        """
+        Explode muti-part geometries into multiple single geometries.
+
+        Each row containing a multi-part geometry will be split into
+        multiple rows with single geometries, thereby increasing the vertical
+        size of the GeoDataFrame.
+
+        The index of the input geodataframe is no longer unique and is
+        replaced with a multi-index (original index with additional level
+        indicating the multiple geometries: a new zero-based index for each
+        single part geometry per multi-part geometry).
+
+        Returns
+        -------
+        GeoDataFrame
+            Exploded geodataframe with each single geometry
+            as a separate entry in the geodataframe.
+
+        """
+        df_copy = self.copy()
+
+        exploded_geom = df_copy.geometry.explode().reset_index(level=-1)
+        exploded_index = exploded_geom.columns[0]
+
+        df = pd.concat(
+            [df_copy.drop(df_copy._geometry_column_name, axis=1),
+             exploded_geom], axis=1)
+        # reset to MultiIndex, otherwise df index is only first level of
+        # exploded GeoSeries index.
+        df.set_index(exploded_index, append=True, inplace=True)
+        df.index.names = list(self.index.names) + [None]
+        geo_df = df.set_geometry(self._geometry_column_name)
+        return geo_df
+
+
 def _dataframe_set_geometry(self, col, drop=False, inplace=False, crs=None):
     if inplace:
         raise ValueError("Can't do inplace setting when converting from"
