@@ -1,10 +1,9 @@
 from __future__ import print_function
-
 from distutils.version import LooseVersion
 import warnings
 
 import numpy as np
-
+import pandas as pd
 
 def _flatten_multi_geoms(geoms, colors=None):
     """
@@ -78,7 +77,12 @@ def plot_polygon_collection(ax, geoms, values=None, color=None,
 
     collection : matplotlib.collections.Collection that was plotted
     """
-    from descartes.patch import PolygonPatch
+
+    try:
+        from descartes.patch import PolygonPatch
+    except ImportError:
+        raise ImportError("The descartes package is required"
+                          " for plotting polygons in geopandas.")
     from matplotlib.collections import PatchCollection
 
     geoms, values = _flatten_multi_geoms(geoms, values)
@@ -164,20 +168,17 @@ def plot_point_collection(ax, geoms, values=None, color=None,
                           cmap=None, vmin=None, vmax=None,
                           marker='o', markersize=None, **kwargs):
     """
-    Plots a collection of Point geometries to `ax`
+    Plots a collection of Point and MultiPoint geometries to `ax`
 
     Parameters
     ----------
-
     ax : matplotlib.axes.Axes
         where shapes will be plotted
-
-    geoms : sequence of `N` Points
+    geoms : sequence of `N` Points or MultiPoints
 
     values : a sequence of `N` values, optional
         Values mapped to colors using vmin, vmax, and cmap.
         Cannot be specified together with `color`.
-
     markersize : scalar or array-like, optional
         Size of the markers. Note that under the hood ``scatter`` is
         used, so the specified value will be proportional to the
@@ -190,8 +191,11 @@ def plot_point_collection(ax, geoms, values=None, color=None,
     if values is not None and color is not None:
         raise ValueError("Can only specify one of 'values' and 'color' kwargs")
 
-    x = geoms.x.values
-    y = geoms.y.values
+    geoms, values = _flatten_multi_geoms(geoms, values)
+    if None in values:
+        values = None
+    x = [p.x for p in geoms]
+    y = [p.y for p in geoms]
 
     # matplotlib 1.4 does not support c=None, and < 2.0 does not support s=None
     if values is not None:
@@ -212,30 +216,25 @@ def plot_series(s, cmap=None, color=None, ax=None, figsize=None, **style_kwds):
 
     Parameters
     ----------
-
     s : Series
-        The GeoSeries to be plotted.  Currently Polygon,
+        The GeoSeries to be plotted. Currently Polygon,
         MultiPolygon, LineString, MultiLineString and Point
         geometries can be plotted.
-
     cmap : str (default None)
-        The name of a colormap recognized by matplotlib.  Any
+        The name of a colormap recognized by matplotlib. Any
         colormap will work, but categorical colormaps are
-        generally recommended.  Examples of useful discrete
+        generally recommended. Examples of useful discrete
         colormaps include:
 
             tab10, tab20, Accent, Dark2, Paired, Pastel1, Set1, Set2
 
     color : str (default None)
         If specified, all objects will be colored uniformly.
-
     ax : matplotlib.pyplot.Artist (default None)
         axes on which to draw the plot
-
     figsize : pair of floats (default None)
         Size of the resulting matplotlib.figure.Figure. If the argument
         ax is given explicitly, figsize is ignored.
-
     **style_kwds : dict
         Color options to be passed on to the actual plot function, such
         as ``edgecolor``, ``facecolor``, ``linewidth``, ``markersize``,
@@ -243,8 +242,7 @@ def plot_series(s, cmap=None, color=None, ax=None, figsize=None, **style_kwds):
 
     Returns
     -------
-
-    matplotlib axes instance
+    ax : matplotlib axes instance
     """
     if 'colormap' in style_kwds:
         warnings.warn("'colormap' is deprecated, please use 'cmap' instead "
@@ -258,7 +256,12 @@ def plot_series(s, cmap=None, color=None, ax=None, figsize=None, **style_kwds):
     import matplotlib.pyplot as plt
     if ax is None:
         fig, ax = plt.subplots(figsize=figsize)
-        ax.set_aspect('equal')
+    ax.set_aspect('equal')
+
+    if s.empty:
+        warnings.warn("The GeoSeries you are attempting to plot is "
+                      "empty. Nothing has been displayed.", UserWarning)
+        return ax
 
     # if cmap is specified, create range of colors based on cmap
     values = None
@@ -274,7 +277,8 @@ def plot_series(s, cmap=None, color=None, ax=None, figsize=None, **style_kwds):
                           | (geom_types == 'MultiPolygon'))
     line_idx = np.asarray((geom_types == 'LineString')
                           | (geom_types == 'MultiLineString'))
-    point_idx = np.asarray(geom_types == 'Point')
+    point_idx = np.asarray((geom_types == 'Point')
+                          | (geom_types == 'MultiPoint'))
 
     # plot all Polygons and all MultiPolygon components in the same collection
     polys = s.geometry[poly_idx]
@@ -309,7 +313,8 @@ def plot_series(s, cmap=None, color=None, ax=None, figsize=None, **style_kwds):
 
 def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
                    categorical=False, legend=False, scheme=None, k=5,
-                   vmin=None, vmax=None, figsize=None, **style_kwds):
+                   vmin=None, vmax=None, markersize=None, figsize=None,
+                   legend_kwds=None, **style_kwds):
     """
     Plot a GeoDataFrame.
 
@@ -319,52 +324,51 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
 
     Parameters
     ----------
-
     df : GeoDataFrame
         The GeoDataFrame to be plotted.  Currently Polygon,
         MultiPolygon, LineString, MultiLineString and Point
         geometries can be plotted.
-
-    column : str (default None)
-        The name of the column to be plotted. Ignored if `color` is also set.
-
+    column : str, np.array, pd.Series (default None)
+        The name of the dataframe column, np.array, or pd.Series to be plotted.
+        If np.array or pd.Series are used then it must have same length as
+        dataframe. Values are used to color the plot. Ignored if `color` is
+        also set.
     cmap : str (default None)
         The name of a colormap recognized by matplotlib.
-
+    color : str (default None)
+        If specified, all objects will be colored uniformly.
+    ax : matplotlib.pyplot.Artist (default None)
+        axes on which to draw the plot
     categorical : bool (default False)
         If False, cmap will reflect numerical values of the
         column being plotted.  For non-numerical columns, this
         will be set to True.
-
-    color : str (default None)
-        If specified, all objects will be colored uniformly.
-
     legend : bool (default False)
         Plot a legend. Ignored if no `column` is given, or if `color` is given.
-
-    ax : matplotlib.pyplot.Artist (default None)
-        axes on which to draw the plot
-
     scheme : str (default None)
         Name of a choropleth classification scheme (requires PySAL).
         A pysal.esda.mapclassify.Map_Classifier object will be used
         under the hood. Supported schemes: 'Equal_interval', 'Quantiles',
         'Fisher_Jenks'
-
-    k   : int (default 5)
+    k : int (default 5)
         Number of classes (ignored if scheme is None)
-
     vmin : None or float (default None)
         Minimum value of cmap. If None, the minimum data value
         in the column to be plotted is used.
-
     vmax : None or float (default None)
         Maximum value of cmap. If None, the maximum data value
         in the column to be plotted is used.
-
-    figsize
+    markersize : str or float or sequence (default None)
+        Only applies to point geometries within a frame.
+        If a str, will use the values in the column of the frame specified
+        by markersize to set the size of markers. Otherwise can be a value
+        to apply to all points, or a sequence of the same length as the
+        number of points.
+    figsize : tuple of integers (default None)
         Size of the resulting matplotlib.figure.Figure. If the argument
         axes is given explicitly, figsize is ignored.
+    legend_kwds : dict (default None)
+        Keyword arguments to pass to ax.legend()
 
     **style_kwds : dict
         Color options to be passed on to the actual plot function, such
@@ -373,7 +377,7 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
 
     Returns
     -------
-    matplotlib axes instance
+    ax : matplotlib axes instance
 
     """
     if 'colormap' in style_kwds:
@@ -384,7 +388,7 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
         warnings.warn("'axes' is deprecated, please use 'ax' instead "
                       "(for consistency with pandas)", FutureWarning)
         ax = style_kwds.pop('axes')
-    if column and color:
+    if column is not None and color is not None:
         warnings.warn("Only specify one of 'column' or 'color'. Using "
                       "'color'.", UserWarning)
         column = None
@@ -392,26 +396,51 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
     import matplotlib
     import matplotlib.pyplot as plt
 
+    if ax is None:
+        fig, ax = plt.subplots(figsize=figsize)
+    ax.set_aspect('equal')
+
+    if df.empty:
+        warnings.warn("The GeoDataFrame you are attempting to plot is "
+                      "empty. Nothing has been displayed.", UserWarning)
+        return ax
+
+    if isinstance(markersize, str):
+        markersize = df[markersize].values
+
     if column is None:
         return plot_series(df.geometry, cmap=cmap, color=color, ax=ax,
-                           figsize=figsize, **style_kwds)
+                           figsize=figsize, markersize=markersize,
+                           **style_kwds)
 
-    if df[column].dtype is np.dtype('O'):
+    # To accept pd.Series and np.arrays as column
+    if isinstance(column, (np.ndarray, pd.Series)):
+        if column.shape[0] != df.shape[0]:
+            raise ValueError("The dataframe and given column have different "
+                             "number of rows.")
+        else:
+            values = np.asarray(column)
+    else:
+        values = np.asarray(df[column])
+
+    if values.dtype is np.dtype('O'):
         categorical = True
 
     # Define `values` as a Series
     if categorical:
         if cmap is None:
-            if LooseVersion(matplotlib.__version__) >= '2.0':
+            if LooseVersion(matplotlib.__version__) >= '2.0.1':
                 cmap = 'tab10'
+            elif LooseVersion(matplotlib.__version__) >= '2.0.0':
+                # Erroneous name.
+                cmap = 'Vega10'
             else:
                 cmap = 'Set1'
-        categories = list(set(df[column].values))
+        categories = list(set(values))
         categories.sort()
         valuemap = dict([(k, v) for (v, k) in enumerate(categories)])
-        values = np.array([valuemap[k] for k in df[column]])
-    else:
-        values = df[column]
+        values = np.array([valuemap[k] for k in values])
+
     if scheme is not None:
         binning = __pysal_choro(values, scheme, k=k)
         # set categorical to True for creating the legend
@@ -421,10 +450,6 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
                       for i in range(len(binedges)-1)]
         values = np.array(binning.yb)
 
-    if ax is None:
-        fig, ax = plt.subplots(figsize=figsize)
-        ax.set_aspect('equal')
-
     mn = values.min() if vmin is None else vmin
     mx = values.max() if vmax is None else vmax
 
@@ -433,7 +458,8 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
                           | (geom_types == 'MultiPolygon'))
     line_idx = np.asarray((geom_types == 'LineString')
                           | (geom_types == 'MultiLineString'))
-    point_idx = np.asarray(geom_types == 'Point')
+    point_idx = np.asarray((geom_types == 'Point')
+                          | (geom_types == 'MultiPoint'))
 
     # plot all Polygons and all MultiPolygon components in the same collection
     polys = df.geometry[poly_idx]
@@ -450,8 +476,11 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
     # plot all Points in the same collection
     points = df.geometry[point_idx]
     if not points.empty:
-        plot_point_collection(ax, points, values[point_idx],
-                              vmin=mn, vmax=mx, cmap=cmap, **style_kwds)
+        if isinstance(markersize, np.ndarray):
+            markersize = markersize[point_idx]
+        plot_point_collection(ax, points, values[point_idx], vmin=mn, vmax=mx,
+                              markersize=markersize, cmap=cmap,
+                              **style_kwds)
 
     if legend and not color:
         from matplotlib.lines import Line2D
@@ -467,10 +496,14 @@ def plot_dataframe(df, column=None, cmap=None, color=None, ax=None,
                     Line2D([0], [0], linestyle="none", marker="o",
                            alpha=style_kwds.get('alpha', 1), markersize=10,
                            markerfacecolor=n_cmap.to_rgba(value)))
-            ax.legend(patches, categories, numpoints=1, loc='best')
+            if legend_kwds is None:
+                legend_kwds = {}
+            legend_kwds.setdefault('numpoints', 1)
+            legend_kwds.setdefault('loc', 'best')
+            ax.legend(patches, categories, **legend_kwds)
         else:
             n_cmap.set_array([])
-            ax.get_figure().colorbar(n_cmap)
+            ax.get_figure().colorbar(n_cmap, ax=ax)
 
     plt.draw()
     return ax
@@ -482,20 +515,16 @@ def __pysal_choro(values, scheme, k=5):
 
     Parameters
     ----------
-
     values
         Series to be plotted
-
-    scheme
-        pysal.esda.mapclassify classificatin scheme
-        ['Equal_interval'|'Quantiles'|'Fisher_Jenks']
-
-    k
+    scheme : str
+        One of pysal.esda.mapclassify classification schemes
+        Options are 'Equal_interval', 'Quantiles', 'Fisher_Jenks'
+    k : int
         number of classes (2 <= k <=9)
 
     Returns
     -------
-
     binning
         Binning objects that holds the Series with values replaced with
         class identifier and the bins.
