@@ -13,11 +13,12 @@ import fiona
 import geopandas
 from geopandas import GeoDataFrame, read_file, GeoSeries
 
-from geopandas.tests.util import (
-    assert_geoseries_equal, connect, create_db, PACKAGE_DIR,
-    validate_boro_df)
-from pandas.util.testing import assert_frame_equal
 import pytest
+from pandas.util.testing import (
+    assert_frame_equal, assert_index_equal, assert_series_equal)
+from geopandas.tests.util import (
+    assert_geoseries_equal, connect, create_postgis, PACKAGE_DIR,
+    validate_boro_df)
 
 
 class TestDataFrame:
@@ -309,6 +310,38 @@ class TestDataFrame:
         assert len(df3) == 2
         assert np.alltrue(df3['Name'].values == self.line_paths)
 
+    def test_to_file_bool(self):
+        """Test error raise when writing with a boolean column (GH #437)."""
+
+        # still want a temp dir in case this test passes
+        tempfilename = os.path.join(self.tempdir, 'boros.shp')
+        df_with_bool = self.df.copy()
+        df_with_bool['bool_column'] = True
+        with pytest.raises(ValueError):
+            df_with_bool.to_file(tempfilename)
+
+    def test_to_file_with_point_z(self):
+        """Test that 3D geometries are retained in writes (GH #612)."""
+
+        tempfilename = os.path.join(self.tempdir, 'test_3Dpoint.shp')
+        point3d = Point(0, 0, 500)
+        point2d = Point(1, 1)
+        df = GeoDataFrame({'a': [1, 2]}, geometry=[point3d, point2d], crs={})
+        df.to_file(tempfilename)
+        df_read = GeoDataFrame.from_file(tempfilename)
+        assert_geoseries_equal(df.geometry, df_read.geometry)
+
+    def test_to_file_with_poly_z(self):
+        """Test that 3D geometries are retained in writes (GH #612)."""
+
+        tempfilename = os.path.join(self.tempdir, 'test_3Dpoly.shp')
+        poly3d = Polygon([[0, 0, 5], [0, 1, 5], [1, 1, 5], [1, 0, 5]])
+        poly2d = Polygon([[0, 0], [0, 1], [1, 1], [1, 0]])
+        df = GeoDataFrame({'a': [1, 2]}, geometry=[poly3d, poly2d], crs={})
+        df.to_file(tempfilename)
+        df_read = GeoDataFrame.from_file(tempfilename)
+        assert_geoseries_equal(df.geometry, df_read.geometry)
+
     def test_to_file_types(self):
         """ Test various integer type columns (GH#93) """
         tempfilename = os.path.join(self.tempdir, 'int.shp')
@@ -328,6 +361,13 @@ class TestDataFrame:
         with pytest.raises(ValueError):
             s.to_file(tempfilename)
 
+    def test_empty_to_file(self):
+        input_empty_df = GeoDataFrame()
+        tempfilename = os.path.join(self.tempdir, 'test.shp')
+        with pytest.raises(
+            ValueError, match="Cannot write empty DataFrame to file."):
+            input_empty_df.to_file(tempfilename)
+
     def test_to_file_schema(self):
         """
         Ensure that the file is written according to the schema
@@ -377,6 +417,14 @@ class TestDataFrame:
         assert all(df2['geometry'].geom_almost_equals(utm['geometry'],
                                                       decimal=2))
 
+    def test_transform_inplace(self):
+        df2 = self.df2.copy()
+        df2.crs = {'init': 'epsg:26918', 'no_defs': True}
+        lonlat = df2.to_crs(epsg=4326)
+        df2.to_crs(epsg=4326, inplace=True)
+        assert all(df2['geometry'].geom_almost_equals(lonlat['geometry'],
+                                                      decimal=2))
+
     def test_to_crs_geo_column_name(self):
         # Test to_crs() with different geometry column name (GH#339)
         df2 = self.df2.copy()
@@ -396,8 +444,7 @@ class TestDataFrame:
             crs = f.crs
 
         df = GeoDataFrame.from_features(features, crs=crs)
-        df.rename(columns=lambda x: x.lower(), inplace=True)
-        validate_boro_df(df)
+        validate_boro_df(df, case_sensitive=True)
         assert df.crs == crs
 
     def test_from_features_unaligned_properties(self):
@@ -449,7 +496,7 @@ class TestDataFrame:
 
     def test_from_postgis_default(self):
         con = connect('test_geopandas')
-        if con is None or not create_db(self.df):
+        if con is None or not create_postgis(self.df):
             raise pytest.skip()
 
         try:
@@ -458,23 +505,21 @@ class TestDataFrame:
         finally:
             con.close()
 
-        validate_boro_df(df)
+        validate_boro_df(df, case_sensitive=False)
 
     def test_from_postgis_custom_geom_col(self):
         con = connect('test_geopandas')
-        if con is None or not create_db(self.df):
+        geom_col = "the_geom"
+        if con is None or not create_postgis(self.df, geom_col=geom_col):
             raise pytest.skip()
 
         try:
-            sql = """SELECT
-                     borocode, boroname, shape_leng, shape_area,
-                     geom AS __geometry__
-                     FROM nybb;"""
-            df = GeoDataFrame.from_postgis(sql, con, geom_col='__geometry__')
+            sql = "SELECT * FROM nybb;"
+            df = GeoDataFrame.from_postgis(sql, con, geom_col=geom_col)
         finally:
             con.close()
 
-        validate_boro_df(df)
+        validate_boro_df(df, case_sensitive=False)
 
     def test_dataframe_to_geodataframe(self):
         df = pd.DataFrame({"A": range(len(self.df)), "location":
@@ -495,9 +540,9 @@ class TestDataFrame:
         assert 'location' in df
 
         # should be a copy
-        df.ix[0, "A"] = 100
-        assert gf.ix[0, "A"] == 0
-        assert gf2.ix[0, "A"] == 0
+        df.loc[0, "A"] = 100
+        assert gf.loc[0, "A"] == 0
+        assert gf2.loc[0, "A"] == 0
 
         with pytest.raises(ValueError):
             df.set_geometry('location', inplace=True)
@@ -526,3 +571,165 @@ class TestDataFrame:
         unpickled = pd.read_pickle(filename)
         assert_frame_equal(self.df, unpickled)
         assert self.df.crs == unpickled.crs
+
+
+def check_geodataframe(df, geometry_column='geometry'):
+    assert isinstance(df, GeoDataFrame)
+    assert isinstance(df.geometry, GeoSeries)
+    assert isinstance(df[geometry_column], GeoSeries)
+    assert df._geometry_column_name == geometry_column
+
+
+class TestConstructor:
+
+    def test_dict(self):
+        data = {"A": range(3), "B": np.arange(3.0),
+                "geometry": [Point(x, x) for x in range(3)]}
+        df = GeoDataFrame(data)
+        check_geodataframe(df)
+
+        # with specifying other kwargs
+        df = GeoDataFrame(data, index=list('abc'))
+        check_geodataframe(df)
+        assert_index_equal(df.index, pd.Index(list('abc')))
+
+        df = GeoDataFrame(data, columns=['B', 'A', 'geometry'])
+        check_geodataframe(df)
+        assert_index_equal(df.columns, pd.Index(['B', 'A', 'geometry']))
+
+        df = GeoDataFrame(data, columns=['A', 'geometry'])
+        check_geodataframe(df)
+        assert_index_equal(df.columns, pd.Index(['A', 'geometry']))
+        assert_series_equal(df['A'], pd.Series(range(3), name='A'))
+
+    def test_dict_of_series(self):
+
+        data = {"A": pd.Series(range(3)), "B": pd.Series(np.arange(3.0)),
+                "geometry": GeoSeries([Point(x, x) for x in range(3)])}
+
+        df = GeoDataFrame(data)
+        check_geodataframe(df)
+
+        df = GeoDataFrame(data, index=pd.Index([1, 2]))
+        check_geodataframe(df)
+        assert_index_equal(df.index, pd.Index([1, 2]))
+        assert df['A'].tolist() == [1, 2]
+
+        # one non-series -> length is not correct
+        data = {"A": pd.Series(range(3)), "B": np.arange(3.0),
+                "geometry": GeoSeries([Point(x, x) for x in range(3)])}
+        with pytest.raises(ValueError):
+            GeoDataFrame(data, index=[1, 2])
+
+    def test_dict_specified_geometry(self):
+
+        data = {"A": range(3), "B": np.arange(3.0),
+                "other_geom": [Point(x, x) for x in range(3)]}
+
+        df = GeoDataFrame(data, geometry='other_geom')
+        check_geodataframe(df, 'other_geom')
+
+        with pytest.raises(ValueError):
+            df = GeoDataFrame(data, geometry='geometry')
+
+        # when no geometry specified -> works but raises error once
+        # trying to access geometry
+        df = GeoDataFrame(data)
+
+        with pytest.raises(AttributeError):
+            _ = df.geometry
+
+        df = df.set_geometry('other_geom')
+        check_geodataframe(df, 'other_geom')
+
+        # combined with custom args
+        df = GeoDataFrame(data, geometry='other_geom',
+                          columns=['B', 'other_geom'])
+        check_geodataframe(df, 'other_geom')
+        assert_index_equal(df.columns, pd.Index(['B', 'other_geom']))
+        assert_series_equal(df['B'], pd.Series(np.arange(3.), name='B'))
+
+        df = GeoDataFrame(data, geometry='other_geom',
+                          columns=['other_geom', 'A'])
+        check_geodataframe(df, 'other_geom')
+        assert_index_equal(df.columns, pd.Index(['other_geom', 'A']))
+        assert_series_equal(df['A'], pd.Series(range(3), name='A'))
+
+    def test_array(self):
+        data = {"A": range(3), "B": np.arange(3.0),
+                "geometry": [Point(x, x) for x in range(3)]}
+        a = np.array([data['A'], data['B'], data['geometry']], dtype=object).T
+
+        df = GeoDataFrame(a, columns=['A', 'B', 'geometry'])
+        check_geodataframe(df)
+
+        df = GeoDataFrame(a, columns=['A', 'B', 'other_geom'],
+                          geometry='other_geom')
+        check_geodataframe(df, 'other_geom')
+
+    def test_from_frame(self):
+        data = {"A": range(3), "B": np.arange(3.0),
+                "geometry": [Point(x, x) for x in range(3)]}
+        gpdf = GeoDataFrame(data)
+        pddf = pd.DataFrame(data)
+        check_geodataframe(gpdf)
+        assert type(pddf) == pd.DataFrame
+
+        for df in [gpdf, pddf]:
+
+            res = GeoDataFrame(df)
+            check_geodataframe(res)
+
+            res = GeoDataFrame(df, index=pd.Index([0, 2]))
+            check_geodataframe(res)
+            assert_index_equal(res.index, pd.Index([0, 2]))
+            assert res['A'].tolist() == [0, 2]
+
+            res = GeoDataFrame(df, columns=['geometry', 'B'])
+            check_geodataframe(res)
+            assert_index_equal(res.columns, pd.Index(['geometry', 'B']))
+
+            with pytest.raises(ValueError):
+                GeoDataFrame(df, geometry='other_geom')
+
+    def test_from_frame_specified_geometry(self):
+        data = {"A": range(3), "B": np.arange(3.0),
+                "other_geom": [Point(x, x) for x in range(3)]}
+
+        gpdf = GeoDataFrame(data, geometry='other_geom')
+        check_geodataframe(gpdf, 'other_geom')
+        pddf = pd.DataFrame(data)
+
+        for df in [gpdf, pddf]:
+            res = GeoDataFrame(df, geometry='other_geom')
+            check_geodataframe(res, 'other_geom')
+
+        # when passing GeoDataFrame with custom geometry name to constructor
+        # an invalid geodataframe is the result TODO is this desired ?
+        df = GeoDataFrame(gpdf)
+        with pytest.raises(AttributeError):
+            df.geometry
+
+    def test_only_geometry(self):
+        df = GeoDataFrame(geometry=[Point(x, x) for x in range(3)])
+        check_geodataframe(df)
+
+        df = GeoDataFrame({'geometry': [Point(x, x) for x in range(3)]})
+        check_geodataframe(df)
+
+        df = GeoDataFrame({'other_geom': [Point(x, x) for x in range(3)]},
+                          geometry='other_geom')
+        check_geodataframe(df, 'other_geom')
+
+    def test_no_geometries(self):
+        # keeps GeoDataFrame class (no DataFrame)
+        data = {"A": range(3), "B": np.arange(3.0)}
+        df = GeoDataFrame(data)
+        assert type(df) == GeoDataFrame
+
+    def test_empty(self):
+        df = GeoDataFrame()
+        assert type(df) == GeoDataFrame
+
+        df = GeoDataFrame({'A': [], 'B': []}, geometry=[])
+        assert type(df) == GeoDataFrame
