@@ -20,6 +20,13 @@ from numpy.testing import assert_array_equal
 from pandas.util.testing import assert_series_equal, assert_frame_equal
 
 
+def assert_array_dtype_equal(a, b, *args, **kwargs):
+    a = np.asanyarray(a)
+    b = np.asanyarray(b)
+    assert a.dtype == b.dtype
+    assert_array_equal(a, b, *args, **kwargs)
+
+
 class TestGeomMethods:
 
     def setup_method(self):
@@ -32,6 +39,7 @@ class TestGeomMethods:
         self.nested_squares = Polygon(self.sq.boundary,
                                       [self.inner_sq.boundary])
         self.p0 = Point(5, 5)
+        self.p3d = Point(5, 5, 5)
         self.g0 = GeoSeries([self.t1, self.t2, self.sq, self.inner_sq,
                              self.nested_squares, self.p0])
         self.g1 = GeoSeries([self.t1, self.sq])
@@ -39,6 +47,8 @@ class TestGeomMethods:
         self.g3 = GeoSeries([self.t1, self.t2])
         self.g3.crs = {'init': 'epsg:4326', 'no_defs': True}
         self.g4 = GeoSeries([self.t2, self.t1])
+        self.g4.crs = {'init': 'epsg:4326', 'no_defs': True}
+        self.g_3d = GeoSeries([self.p0, self.p3d])
         self.na = GeoSeries([self.t1, self.t2, Polygon()])
         self.na_none = GeoSeries([self.t1, None])
         self.a1 = self.g1.copy()
@@ -53,6 +63,9 @@ class TestGeomMethods:
         self.l2 = LineString([(0, 0), (1, 0), (1, 1), (0, 1)])
         self.g5 = GeoSeries([self.l1, self.l2])
         self.g6 = GeoSeries([self.p0, self.t3])
+        self.empty = GeoSeries([])
+        self.empty.crs = {'init': 'epsg:4326', 'no_defs': True}
+        self.empty_poly = Polygon()
 
         # Crossed lines
         self.l3 = LineString([(0, 0), (1, 1)])
@@ -186,9 +199,19 @@ class TestGeomMethods:
         result = getattr(gdf, op)
         fcmp(result, expected)
 
+    def test_crs_warning(self):
+        # operations on geometries should warn for different CRS
+        no_crs_g3 = self.g3.copy()
+        no_crs_g3.crs = None
+        with pytest.warns(UserWarning):
+            self._test_binary_topological('intersection', self.g3,
+                                          self.g3, no_crs_g3)
+
     def test_intersection(self):
         self._test_binary_topological('intersection', self.t1,
                                       self.g1, self.g2)
+        self._test_binary_topological('intersection', self.empty_poly,
+                                      self.g1, self.empty)
 
     def test_union_series(self):
         self._test_binary_topological('union', self.sq, self.g1, self.g2)
@@ -253,7 +276,7 @@ class TestGeomMethods:
 
     def test_contains(self):
         expected = [True, False, True, False, False, False]
-        assert_array_equal(expected, self.g0.contains(self.t1))
+        assert_array_dtype_equal(expected, self.g0.contains(self.t1))
 
     def test_length(self):
         expected = Series(np.array([2 + np.sqrt(2), 4]), index=self.g1.index)
@@ -266,48 +289,58 @@ class TestGeomMethods:
 
     def test_crosses(self):
         expected = [False, False, False, False, False, False]
-        assert_array_equal(expected, self.g0.crosses(self.t1))
+        assert_array_dtype_equal(expected, self.g0.crosses(self.t1))
 
         expected = [False, True]
-        assert_array_equal(expected, self.crossed_lines.crosses(self.l3))
+        assert_array_dtype_equal(expected, self.crossed_lines.crosses(self.l3))
 
     def test_disjoint(self):
         expected = [False, False, False, False, False, True]
-        assert_array_equal(expected, self.g0.disjoint(self.t1))
+        assert_array_dtype_equal(expected, self.g0.disjoint(self.t1))
 
     def test_distance(self):
         expected = Series(np.array([np.sqrt((5 - 1)**2 + (5 - 1)**2), np.nan]),
                           self.na_none.index)
-        assert_array_equal(expected, self.na_none.distance(self.p0))
+        assert_array_dtype_equal(expected, self.na_none.distance(self.p0))
 
         expected = Series(np.array([np.sqrt(4**2 + 4**2), np.nan]),
                           self.g6.index)
-        assert_array_equal(expected, self.g6.distance(self.na_none))
+        assert_array_dtype_equal(expected, self.g6.distance(self.na_none))
 
     def test_intersects(self):
         expected = [True, True, True, True, True, False]
-        assert_array_equal(expected, self.g0.intersects(self.t1))
+        assert_array_dtype_equal(expected, self.g0.intersects(self.t1))
 
         expected = [True, False]
-        assert_array_equal(expected, self.na_none.intersects(self.t2))
+        assert_array_dtype_equal(expected, self.na_none.intersects(self.t2))
+
+        expected = np.array([], dtype=bool)
+        assert_array_dtype_equal(expected, self.empty.intersects(self.t1))
+
+        expected = np.array([], dtype=bool)
+        assert_array_dtype_equal(
+                expected, self.empty.intersects(self.empty_poly))
+
+        expected = [False] * 6
+        assert_array_dtype_equal(expected, self.g0.intersects(self.empty_poly))
 
     def test_overlaps(self):
         expected = [True, True, False, False, False, False]
-        assert_array_equal(expected, self.g0.overlaps(self.inner_sq))
+        assert_array_dtype_equal(expected, self.g0.overlaps(self.inner_sq))
 
         expected = [False, False]
-        assert_array_equal(expected, self.g4.overlaps(self.t1))
+        assert_array_dtype_equal(expected, self.g4.overlaps(self.t1))
 
     def test_touches(self):
         expected = [False, True, False, False, False, False]
-        assert_array_equal(expected, self.g0.touches(self.t1))
+        assert_array_dtype_equal(expected, self.g0.touches(self.t1))
 
     def test_within(self):
         expected = [True, False, False, False, False, False]
-        assert_array_equal(expected, self.g0.within(self.t1))
+        assert_array_dtype_equal(expected, self.g0.within(self.t1))
 
         expected = [True, True, True, True, True, False]
-        assert_array_equal(expected, self.g0.within(self.sq))
+        assert_array_dtype_equal(expected, self.g0.within(self.sq))
 
     def test_is_valid(self):
         expected = Series(np.array([True] * len(self.g1)), self.g1.index)
@@ -325,12 +358,16 @@ class TestGeomMethods:
         expected = Series(np.array([True] * len(self.g1)), self.g1.index)
         self._test_unary_real('is_simple', expected, self.g1)
 
+    def test_has_z(self):
+        expected = Series([False, True], self.g_3d.index)
+        self._test_unary_real('has_z', expected, self.g_3d)
+
     def test_xy_points(self):
         expected_x = [-73.9847, -74.0446]
         expected_y = [40.7484, 40.6893]
 
-        assert_array_equal(expected_x, self.landmarks.geometry.x)
-        assert_array_equal(expected_y, self.landmarks.geometry.y)
+        assert_array_dtype_equal(expected_x, self.landmarks.geometry.x)
+        assert_array_dtype_equal(expected_y, self.landmarks.geometry.y)
 
     def test_xy_polygons(self):
         # accessing x attribute in polygon geoseries should raise an error
@@ -340,6 +377,18 @@ class TestGeomMethods:
         with pytest.raises(ValueError):
             _ = self.gdf1.geometry.y
 
+    def test_centroid(self):
+        polygon = Polygon([(-1, -1), (1, -1), (1, 1), (-1, 1)])
+        point = Point(0, 0)
+        polygons = GeoSeries([polygon for i in range(3)])
+        points = GeoSeries([point for i in range(3)])
+        assert_geoseries_equal(polygons.centroid, points)
+
+    def test_convex_hull(self):
+        # the convex hull of a square should be the same as the square
+        squares = GeoSeries([self.sq for i in range(3)])
+        assert_geoseries_equal(squares, squares.convex_hull)
+
     def test_exterior(self):
         exp_exterior = GeoSeries([LinearRing(p.boundary) for p in self.g3])
         for expected, computed in zip(exp_exterior, self.g3.exterior):
@@ -423,6 +472,20 @@ class TestGeomMethods:
         res = res.skew(ys=-skew, origin=o)
         assert geom_almost_equals(expected, res)
 
+    def test_buffer(self):
+        original = GeoSeries([Point(0, 0)])
+        expected = GeoSeries([Polygon(((5, 0), (0, -5), (-5, 0), (0, 5),
+                                       (5, 0)))])
+        calculated = original.buffer(5, resolution=1)
+        assert geom_almost_equals(expected, calculated)
+
+    def test_buffer_args(self):
+        args = dict(cap_style=3, join_style=2, mitre_limit=2.5)
+        calculated_series = self.g0.buffer(10, **args)
+        for original, calculated in zip(self.g0, calculated_series):
+            expected = original.buffer(10, **args)
+            assert calculated.equals(expected)
+
     def test_envelope(self):
         e = self.g3.envelope
         assert np.all(e.geom_equals(self.sq))
@@ -438,19 +501,33 @@ class TestGeomMethods:
                            'col1': range(len(self.landmarks))})
         assert tuple(df.total_bounds) == bbox
 
-    def test_explode(self):
+    def test_explode_geoseries(self):
         s = GeoSeries([MultiPoint([(0, 0), (1, 1)]),
                        MultiPoint([(2, 2), (3, 3), (4, 4)])])
-
+        s.index.name = 'test_index_name'
+        expected_index_name = ['test_index_name', None]
         index = [(0, 0), (0, 1), (1, 0), (1, 1), (1, 2)]
         expected = GeoSeries([Point(0, 0), Point(1, 1), Point(2, 2),
                               Point(3, 3), Point(4, 4)],
-                             index=MultiIndex.from_tuples(index))
-
+                             index=MultiIndex.from_tuples(
+                                index, names=expected_index_name))
         assert_geoseries_equal(expected, s.explode())
 
-        df = self.gdf1[:2].set_geometry(s)
-        assert_geoseries_equal(expected, df.explode())
+    @pytest.mark.parametrize("index_name", [None, 'test'])
+    def test_explode_geodataframe(self, index_name):
+        s = GeoSeries([MultiPoint([Point(1, 2), Point(2, 3)]), Point(5, 5)])
+        df = GeoDataFrame({'col': [1, 2], 'geometry': s})
+        df.index.name = index_name
+
+        test_df = df.explode()
+
+        expected_s = GeoSeries([Point(1, 2), Point(2, 3), Point(5, 5)])
+        expected_df = GeoDataFrame({'col': [1, 1, 2], 'geometry': expected_s})
+        expected_index = MultiIndex(levels=[[0, 1], [0, 1]],
+                                    labels=[[0, 0, 1], [0, 1, 0]],
+                                    names=[index_name, None])
+        expected_df = expected_df.set_index(expected_index)
+        assert_frame_equal(test_df, expected_df)
 
     #
     # Test '&', '|', '^', and '-'
