@@ -9,7 +9,7 @@ import matplotlib
 matplotlib.use('Agg')
 import matplotlib.pyplot as plt
 from shapely.affinity import rotate
-from shapely.geometry import MultiPolygon, Polygon, LineString, Point
+from shapely.geometry import MultiPolygon, Polygon, LineString, Point, MultiPoint
 
 from geopandas import GeoSeries, GeoDataFrame, read_file
 
@@ -34,9 +34,15 @@ class TestPointPlotting:
     def setup_method(self):
         self.N = 10
         self.points = GeoSeries(Point(i, i) for i in range(self.N))
+
         values = np.arange(self.N)
         self.df = GeoDataFrame({'geometry': self.points, 'values': values})
 
+        multipoint1 = MultiPoint(self.points)
+        multipoint2 = rotate(multipoint1, 90)
+        self.df2 = GeoDataFrame({'geometry': [multipoint1, multipoint2],
+                                 'values': [0, 1]})
+
     def test_figsize(self):
 
         ax = self.points.plot(figsize=(1, 1))
@@ -105,7 +111,7 @@ class TestPointPlotting:
             ax = self.df.plot(column='values', color='green')
             _check_colors(self.N, ax.collections[0].get_facecolors(), ['green']*self.N)
 
-    def test_style_kwargs(self):
+    def test_markersize(self):
 
         ax = self.points.plot(markersize=10)
         assert ax.collections[0].get_sizes() == [10]
@@ -116,6 +122,17 @@ class TestPointPlotting:
         ax = self.df.plot(column='values', markersize=10)
         assert ax.collections[0].get_sizes() == [10]
 
+        ax = self.df.plot(markersize='values')
+        assert (ax.collections[0].get_sizes() == self.df['values']).all()
+
+        ax = self.df.plot(column='values', markersize='values')
+        assert (ax.collections[0].get_sizes() == self.df['values']).all()
+
+    def test_style_kwargs(self):
+
+        ax = self.points.plot(edgecolors='k')
+        assert (ax.collections[0].get_edgecolor() == [0, 0, 0, 1]).all()
+
     def test_legend(self):
         with warnings.catch_warnings(record=True) as _:  # don't print warning
             # legend ignored if color is given.
@@ -146,6 +163,29 @@ class TestPointPlotting:
         # last point == top of colorbar
         np.testing.assert_array_equal(point_colors[-1], cbar_colors[-1])
 
+    def test_empty_plot(self):
+        s = GeoSeries([])
+        with pytest.warns(UserWarning):
+            ax = s.plot()
+        assert len(ax.collections) == 0
+        df = GeoDataFrame([])
+        with pytest.warns(UserWarning):
+            ax = df.plot()
+        assert len(ax.collections) == 0
+
+    def test_multipoints(self):
+
+        # MultiPoints
+        ax = self.df2.plot()
+        _check_colors(4, ax.collections[0].get_facecolors(),
+                      [MPL_DFT_COLOR] * 4)
+
+
+        ax = self.df2.plot(column='values')
+        cmap = plt.get_cmap()
+        expected_colors = [cmap(0)]* self.N + [cmap(1)] * self.N
+        _check_colors(2, ax.collections[0].get_facecolors(),
+                      expected_colors)
 
 class TestPointZPlotting:
 
@@ -184,24 +224,26 @@ class TestLineStringPlotting:
             _check_colors(self.N, ax.collections[0].get_colors(), ['green']*self.N)
 
     def test_style_kwargs(self):
-
         # linestyle (style patterns depend on linewidth, therefore pin to 1)
         linestyle = 'dashed'
-        ax = self.lines.plot(linestyle=linestyle, linewidth=1)
-        exp_ls = _style_to_linestring_onoffseq(linestyle)
-        for ls in ax.collections[0].get_linestyles():
-            assert ls[0] == exp_ls[0]
-            assert tuple(ls[1]) == exp_ls[1]
+        linewidth = 1
 
-        ax = self.df.plot(linestyle=linestyle, linewidth=1)
+        ax = self.lines.plot(linestyle=linestyle, linewidth=linewidth)
+        exp_ls = _style_to_linestring_onoffseq(linestyle, linewidth)
         for ls in ax.collections[0].get_linestyles():
             assert ls[0] == exp_ls[0]
-            assert tuple(ls[1]) == exp_ls[1]
+            assert ls[1] == exp_ls[1]
 
-        ax = self.df.plot(column='values', linestyle=linestyle, linewidth=1)
+        ax = self.df.plot(linestyle=linestyle, linewidth=linewidth)
         for ls in ax.collections[0].get_linestyles():
             assert ls[0] == exp_ls[0]
-            assert tuple(ls[1]) == exp_ls[1]
+            assert ls[1] == exp_ls[1]
+
+        ax = self.df.plot(column='values', linestyle=linestyle,
+                          linewidth=linewidth)
+        for ls in ax.collections[0].get_linestyles():
+            assert ls[0] == exp_ls[0]
+            assert ls[1] == exp_ls[1]
 
 
 class TestPolygonPlotting:
@@ -272,6 +314,12 @@ class TestPolygonPlotting:
         _check_colors(2, ax.collections[0].get_facecolors(), ['g'] * 2, alpha=0.4)
         _check_colors(2, ax.collections[0].get_edgecolors(), ['r'] * 2, alpha=0.4)
 
+    def test_legend_kwargs(self):
+
+        ax = self.df.plot(column='values', categorical=True, legend=True,
+                          legend_kwds={'frameon': False})
+        assert ax.get_legend().get_frame_on() is False
+
     def test_multipolygons(self):
 
         # MultiPolygons
@@ -469,10 +517,10 @@ class TestPlotCollections:
         # pass through of kwargs
         coll = plot_linestring_collection(ax, self.lines, linestyle='--',
                                           linewidth=1)
-        exp_ls = _style_to_linestring_onoffseq('dashed')
+        exp_ls = _style_to_linestring_onoffseq('dashed', 1)
         res_ls = coll.get_linestyle()[0]
         assert res_ls[0] == exp_ls[0]
-        assert tuple(res_ls[1]) == exp_ls[1]
+        assert res_ls[1] == exp_ls[1]
         ax.cla()
 
     def test_linestrings_values(self):
@@ -584,6 +632,42 @@ class TestPlotCollections:
         ax.cla()
 
 
+def test_column_values():
+    """
+    Check that the dataframe plot method returns same values with an
+    input string (column in df), pd.Series, or np.array
+    """
+    # Build test data
+    t1 = Polygon([(0, 0), (1, 0), (1, 1)])
+    t2 = Polygon([(1, 0), (2, 0), (2, 1)])
+    polys = GeoSeries([t1, t2], index=list('AB'))
+    df = GeoDataFrame({'geometry': polys, 'values': [0, 1]})
+
+    # Test with continous values
+    ax = df.plot(column='values')
+    colors = ax.collections[0].get_facecolors()
+    ax = df.plot(column=df['values'])
+    colors_series = ax.collections[0].get_facecolors()
+    np.testing.assert_array_equal(colors, colors_series)
+    ax = df.plot(column=df['values'].values)
+    colors_array = ax.collections[0].get_facecolors()
+    np.testing.assert_array_equal(colors, colors_array)
+
+    # Test with categorical values
+    ax = df.plot(column='values', categorical=True)
+    colors = ax.collections[0].get_facecolors()
+    ax = df.plot(column=df['values'], categorical=True)
+    colors_series = ax.collections[0].get_facecolors()
+    np.testing.assert_array_equal(colors, colors_series)
+    ax = df.plot(column=df['values'].values, categorical=True)
+    colors_array = ax.collections[0].get_facecolors()
+    np.testing.assert_array_equal(colors, colors_array)
+
+    # Check raised error: is df rows number equal to column legth?
+    with pytest.raises(ValueError, match="different number of rows"):
+        ax = df.plot(column=np.array([1, 2, 3]))
+
+
 def _check_colors(N, actual_colors, expected_colors, alpha=None):
     """
     Asserts that the members of `collection` match the `expected_colors`
@@ -603,7 +687,7 @@ def _check_colors(N, actual_colors, expected_colors, alpha=None):
     expected_colors : sequence of RGBA tuples
     alpha : float (optional)
         If set, this alpha transparency will be applied to the
-        `expected_colors`. (Any transparency on the `collecton` is assumed
+        `expected_colors`. (Any transparency on the `collection` is assumed
         to be set in its own facecolor RGBA tuples.)
     """
     import matplotlib.colors as colors
@@ -619,14 +703,15 @@ def _check_colors(N, actual_colors, expected_colors, alpha=None):
             '{} != {}'.format(actual, conv.to_rgba(expected, alpha=alpha))
 
 
-def _style_to_linestring_onoffseq(linestyle):
+def _style_to_linestring_onoffseq(linestyle, linewidth):
     """ Converts a linestyle string representation, namely one of:
             ['dashed',  'dotted', 'dashdot', 'solid'],
         documented in `Collections.set_linestyle`,
         to the form `onoffseq`.
     """
     if LooseVersion(matplotlib.__version__) >= '2.0':
-        return matplotlib.lines._get_dash_pattern(linestyle)
+        offset, dashes = matplotlib.lines._get_dash_pattern(linestyle)
+        return matplotlib.lines._scale_dashes(offset, dashes, linewidth)
     else:
         from matplotlib.backend_bases import GraphicsContextBase
         return GraphicsContextBase.dashd[linestyle]
