@@ -1,3 +1,5 @@
+from warnings import warn
+
 import numpy as np
 import pandas as pd
 from shapely import prepared
@@ -30,16 +32,16 @@ def sjoin(left_df, right_df, how='inner', op='intersects',
 
     allowed_hows = ['left', 'right', 'inner']
     if how not in allowed_hows:
-        raise ValueError("`how` was \"%s\" but is expected to be in %s" % \
-            (how, allowed_hows))
+        raise ValueError("`how` was \"%s\" but is expected to be in %s" %
+                         (how, allowed_hows))
 
     allowed_ops = ['contains', 'within', 'intersects']
     if op not in allowed_ops:
-        raise ValueError("`op` was \"%s\" but is expected to be in %s" % \
-            (op, allowed_ops))
+        raise ValueError("`op` was \"%s\" but is expected to be in %s" %
+                         (op, allowed_ops))
 
     if left_df.crs != right_df.crs:
-        print('Warning: CRS does not match!')
+        warn('CRS of frames being joined does not match!')
 
     index_left = 'index_%s' % lsuffix
     index_right = 'index_%s' % rsuffix
@@ -54,9 +56,11 @@ def sjoin(left_df, right_df, how='inner', op='intersects',
     # index in geopandas may be any arbitrary dtype. so reset both indices now
     # and store references to the original indices, to be reaffixed later.
     # GH 352
-    left_df.index.name = index_left
-    right_df.index.name = index_right
+    left_df = left_df.copy(deep=True)
+    left_df.index = left_df.index.rename(index_left)
     left_df = left_df.reset_index()
+    right_df = right_df.copy(deep=True)
+    right_df.index = right_df.index.rename(index_right)
     right_df = right_df.reset_index()
 
     if op == "within":
@@ -110,7 +114,7 @@ def sjoin(left_df, right_df, how='inner', op='intersects',
 
     else:
         # when output from the join has no overlapping geometries
-        result = pd.DataFrame(columns=['_key_left', '_key_right'])
+        result = pd.DataFrame(columns=['_key_left', '_key_right'], dtype=float)
 
     if op == "within":
         # within implemented as the inverse of contains; swap names
