@@ -68,6 +68,14 @@ def dfs(request):
 @pytest.mark.skipif(not base.HAS_SINDEX, reason='Rtree absent, skipping')
 class TestSpatialJoin:
 
+    @pytest.mark.parametrize('dfs', ['default-index', 'string-index'],
+                             indirect=True)
+    def test_crs_mismatch(self, dfs):
+        index, df1, df2, expected = dfs
+        df1.crs = {'init': 'epsg:4326', 'no_defs': True}
+        with pytest.warns(UserWarning):
+            sjoin(df1, df2)
+
     @pytest.mark.parametrize('dfs', ['default-index', 'string-index'],
                              indirect=True)
     @pytest.mark.parametrize('op', ['intersects', 'contains', 'within'])
@@ -109,6 +117,23 @@ class TestSpatialJoin:
 
         assert_frame_equal(res, exp)
 
+    def test_empty_join(self):
+        # Check empty joins
+        polygons = geopandas.GeoDataFrame({'col2': [1, 2], 
+                                           'geometry':  [Polygon([(0, 0), (1, 0), 
+                                                                  (1, 1), (0, 1)]), 
+                                                         Polygon([(1, 0), (2, 0), 
+                                                                  (2, 1), (1, 1)])
+                                                        ]})
+        not_in = geopandas.GeoDataFrame({'col1': [1], 
+                             'geometry': [Point(-0.5, 0.5)]})
+        empty = sjoin(not_in, polygons, how='left', op='intersects')
+        assert empty.index_right.isnull().all()
+        empty = sjoin(not_in, polygons, how='right', op='intersects')
+        assert empty.index_left.isnull().all()
+        empty = sjoin(not_in, polygons, how='inner', op='intersects')
+        assert empty.empty
+
     @pytest.mark.parametrize('dfs', ['default-index', 'string-index'],
                              indirect=True)
     @pytest.mark.parametrize('op', ['intersects', 'contains', 'within'])
@@ -182,12 +207,12 @@ class TestSpatialJoinNYBB:
         # points within polygons
         df = sjoin(self.pointdf, self.polydf, how="left", op="within")
         assert df.shape == (21, 8)
-        assert df.ix[1]['BoroName'] == 'Staten Island'
+        assert df.loc[1]['BoroName'] == 'Staten Island'
 
         # points contain polygons? never happens so we should have nulls
         df = sjoin(self.pointdf, self.polydf, how="left", op="contains")
         assert df.shape == (21, 8)
-        assert np.isnan(df.ix[1]['Shape_Area'])
+        assert np.isnan(df.loc[1]['Shape_Area'])
 
     def test_sjoin_bad_op(self):
         # AttributeError: 'Point' object has no attribute 'spandex'
@@ -200,6 +225,15 @@ class TestSpatialJoinNYBB:
         assert 'Shape_Area_left' in df.columns
         assert 'Shape_Area_right' in df.columns
 
+    @pytest.mark.parametrize('how', ['left', 'right', 'inner'])
+    def test_sjoin_named_index(self, how):
+        #original index names should be unchanged
+        pointdf2 = self.pointdf.copy()
+        pointdf2.index.name = 'pointid'
+        df = sjoin(pointdf2, self.polydf, how=how)
+        assert pointdf2.index.name == 'pointid'
+        assert self.polydf.index.name == None
+
     def test_sjoin_values(self):
         # GH190
         self.polydf.index = [1, 3, 4, 5, 6]
@@ -265,3 +299,21 @@ class TestSpatialJoinNYBB:
     def test_sjoin_outer(self):
         df = sjoin(self.pointdf, self.polydf, how="outer")
         assert df.shape == (21, 8)
+
+
+@pytest.mark.skipif(not base.HAS_SINDEX, reason='Rtree absent, skipping')
+class TestSpatialJoinNaturalEarth:
+
+    def setup_method(self):
+        world_path = geopandas.datasets.get_path("naturalearth_lowres")
+        cities_path = geopandas.datasets.get_path("naturalearth_cities")
+        self.world = read_file(world_path)
+        self.cities = read_file(cities_path)
+
+    def test_sjoin_inner(self):
+        # GH637
+        countries = self.world[["geometry", "name"]]
+        countries = countries.rename(columns={"name": "country"})
+        cities_with_country = sjoin(self.cities, countries, how="inner",
+                                    op="intersects")
+        assert cities_with_country.shape == (172, 4)
